var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
import { jsx as _jsx } from "react/jsx-runtime";
import { useContext, useEffect, useRef, useState } from 'react';
import SUIContext from '@s-ui/react-context';
// used to store the last definition of ClientPage component so it can be reused
var latestClientPage;
var getInitialPropsFromWindow = function () {
    // if no window initial props, then do nothing
    if (typeof window.__INITIAL_PROPS__ === 'undefined')
        return;
    // make a copy of the content safely
    var windowInitialProps = __assign({}, window.__INITIAL_PROPS__);
    // remove the variable of the window
    window.__INITIAL_PROPS__ = undefined;
    // return retrieved props from window
    return windowInitialProps;
};
// extract needed info from props for routeInfo object
var createRouteInfoFromProps = function (_a) {
    var location = _a.location, params = _a.params, routes = _a.routes;
    return ({
        location: location,
        params: params,
        routes: routes
    });
};
// This HoC creates the PageComponent prepared for the Client Side Rendering
// It renders a placeholder, if any specified as `renderLoading` on the PageComponent.
// Also, executes the getInitialProps to retrieve the initialProps and then
// renders the component with the initialProps, hiding the placeholder.
//
// Additionally, if `keepMounted` on the PageComponent is true, it keeps the
// component alive by running getInitialProps again when a route change is done
// and the very same component is matched by the router, so PageComponent just
// gets props updated. Also, since PageComponent keeps mounted it will receive
// an `isLoading` prop while getInitialProps is in progress.
export default (function (Page) {
    var _a, _b;
    // gather window initial props for this Page, if present
    var windowInitialProps = getInitialPropsFromWindow();
    var _c = Page.keepMounted, keepMounted = _c === void 0 ? false : _c;
    // define Page wrapper component
    var ClientPage = function (props) {
        var initialPropsFromWindowRef = useRef(windowInitialProps);
        // used to know if initialProps has been requested at least once
        var requestedInitialPropsOnceRef = useRef(!(windowInitialProps == null));
        // create routeInfo object from current props which are updated
        var routeInfo = createRouteInfoFromProps(props);
        // consume sui context from the context provider
        var suiContext = useContext(SUIContext);
        // pathName from context is outdated, so we update it from routeInfo
        var context = __assign(__assign({}, suiContext), { pathName: routeInfo.location.pathname });
        var _a = useState(function () { return ({
            initialProps: (initialPropsFromWindowRef.current != null) || {},
            isLoading: initialPropsFromWindowRef.current == null
        }); }), _b = _a[0], initialProps = _b.initialProps, isLoading = _b.isLoading, setState = _a[1];
        useEffect(function () {
            // check if got initial props from window, because then there's no need
            // to request them again from client
            if (initialPropsFromWindowRef.current != null) {
                initialPropsFromWindowRef.current = undefined;
            }
            else {
                // only update state if already request initial props
                if (requestedInitialPropsOnceRef.current) {
                    setState({ initialProps: initialProps, isLoading: true });
                }
                Page.getInitialProps({ context: context, routeInfo: routeInfo })
                    .then(function (initialProps) {
                    setState({ initialProps: initialProps, isLoading: false });
                })
                    .catch(function (error) {
                    setState({ initialProps: { error: error }, isLoading: false });
                })
                    .finally(function () {
                    if (requestedInitialPropsOnceRef.current)
                        return;
                    requestedInitialPropsOnceRef.current = true;
                });
            }
        }, [routeInfo.location]); // eslint-disable-line react-hooks/exhaustive-deps
        var renderPage = function () { return (_jsx(Page, __assign({}, initialProps, props, { isLoading: isLoading }), void 0)); };
        // if the page has a `keepMounted` property and already requested
        // initialProps once, just keep rendering the page
        if (keepMounted && requestedInitialPropsOnceRef.current) {
            return renderPage();
        }
        var renderLoading = function () {
            // check if the page has a `renderLoading` method, if not, just render nothing
            return (Page.renderLoading != null)
                ? Page.renderLoading({ context: context, routeInfo: routeInfo })
                : null;
        };
        return isLoading ? renderLoading() : renderPage();
    };
    // if `keepMounted` property is found and the component is the same one,
    // we just reuse it instead of returning a new one
    if (keepMounted &&
        Page.displayName === ((_a = latestClientPage === null || latestClientPage === void 0 ? void 0 : latestClientPage.Page) === null || _a === void 0 ? void 0 : _a.displayName)) {
        return latestClientPage;
    }
    var displayName = (_b = Page.displayName) !== null && _b !== void 0 ? _b : 'WithoutDisplayName';
    // save original page component
    ClientPage.Page = Page;
    // add ClientPage to name of the component
    ClientPage.displayName = "ClientPage(" + displayName + ")";
    // save ClientPage to latestClientPage
    latestClientPage = ClientPage;
    // return the page
    return ClientPage;
});
